# frozen_string_literal: true

require 'open3'
require 'digest'

class PurgecssNotFoundError < RuntimeError; end
class PurgecssRuntimeError < RuntimeError; end

Jekyll::Hooks.register(:site, :post_write) do |site|
  next unless Jekyll.env == 'production'

  raise PurgecssNotFoundError unless File.file?('./node_modules/.bin/purgecss')

  PURGECSS = File.join('node_modules', '.bin', 'purgecss')

  Dir.glob(File.join(site.dest, '**', '*.css')).each do |css|
    digest_filename = Digest::SHA256.hexdigest(css)
    digest_file     = File.join(site.cache_dir, digest_filename)
    current_digest  = Digest::SHA256.file(css).hexdigest
    previous_digest = File.read(digest_file) if File.exist?(digest_file)

    if current_digest == previous_digest
      Jekyll.logger.info 'PurgeCSS:', "Already purged #{css}"
    else
      Jekyll.logger.info 'PurgeCSS:', "Purging #{css}"

      Open3.popen2e(
        PURGECSS,
        '--css', css,
        '--content', File.join(site.dest, '**', '*.html'), File.join(site.dest, '*.html'),
        '--out', File.dirname(css),
        chdir: site.source) do |_, o, t|

        Thread.new do
          o.each do |line|
            Jekyll.logger.info 'PurgeCSS:', line
          end
        rescue IOError => e
          Jekyll.logger.error 'PurgeCSS:', e.message
        end

        raise PurgecssRuntimeError unless t.value.success?
      end

      File.write(digest_file, 'w') do |f|
        f.write current_digest
      end
    end
  end
end
